@@ -1,4 +1,3 @@
-
 package IO::Handle;
 
 =head1 NAME
@@ -20,6 +19,7 @@ IO::Handle - supply object methods for I/O handles
         $io->print("Some text\n");
     }
 
+    # setvbuf is not available by default on Perls 5.8.0 and later.
     use IO::Handle '_IOLBF';
     $io->setvbuf($buffer_var, _IOLBF, 1024);
 
@@ -193,6 +193,10 @@ C<_IOLBF>, and C<_IONBF> for setvbuf()--except that the buffer parameter
 specifies a scalar variable to use as a buffer. You should only
 change the buffer before any I/O, or immediately after calling flush.
 
+WARNING: The IO::Handle::setvbuf() is not available by default on
+Perls 5.8.0 and later because setvbuf() is rather specific to using
+the stdio library, while Perl prefers the new perlio subsystem instead.
+
 WARNING: A variable used as a buffer by C<setbuf> or C<setvbuf> B<must not
 be modified> in any way until the IO::Handle is closed or C<setbuf> or
 C<setvbuf> is called again, or memory corruption may result! Remember that
@@ -258,7 +262,8 @@ use IO ();	# Load the XS module
 require Exporter;
 @ISA = qw(Exporter);
 
-$VERSION = "1.21_00";
+$VERSION = "1.23";
+$VERSION = eval $VERSION;
 
 @EXPORT_OK = qw(
     autoflush
