@@ -403,10 +403,10 @@ libswanted_uselargefiles="`getconf XBS5_ILP32_OFFBIG_LIBS 2>/dev/null|sed -e 's@
 	esac
 	case "$gccversion" in
 	'') ;;
-	*) # Remove xlc-spefific -qflags.
+	*) # Remove xlc-specific -qflags.
 	   ccflags="`echo $ccflags | sed -e 's@ -q[^ ]*@ @g' -e 's@^-q[^ ]* @@g'`"
 	   ldflags="`echo $ldflags | sed -e 's@ -q[^ ]*@ @g' -e 's@^-q[^ ]* @@g'`"
-	   # Move xld-spefific -bflags.
+	   # Move xlc-specific -bflags.
 	   ccflags="`echo $ccflags | sed -e 's@ -b@ -Wl,-b@g'`"
 	   ldflags="`echo ' '$ldflags | sed -e 's@ -b@ -Wl,-b@g'`"
 	   lddlflags="`echo ' '$lddlflags | sed -e 's@ -b@ -Wl,-b@g'`"
