@@ -112,7 +112,7 @@ then
 else
     ###############################################################
     # Need this in release 5 because of changed fpu exeption rules
-    ccflags="$ccflags -D PERL_SCO5"
+    ccflags="$ccflags -D HAS_FPSETMASK"
 
     ###############################################################
     # In Release 5, always compile ELF objects
@@ -180,6 +180,13 @@ set X $libswanted
 shift
 libswanted="$*"
 
+###############################################################
+# Remove libbind because it conflicts with libsocket.
+libswanted=`echo " $libswanted " | sed -e 's/ bind / /'`
+set X $libswanted
+shift
+libswanted="$*"
+
 ###############################################################
 # Try to use libintl.a since it has strcoll and strxfrm
 libswanted="intl $libswanted"
