@@ -15,8 +15,14 @@
 
 package TestInit;
 
+$VERSION = 1.01;
+
 chdir 't' if -d 't';
 @INC = '../lib';
+
+# Don't interfere with the taintedness of %ENV, this could perturbate tests
+$ENV{PERL_CORE} = 1 unless ${^TAINT};
+
 $0 =~ s/\.dp$//; # for the test.deparse make target
 1;
 
